@@ -21,6 +21,7 @@ package com.eteks.sweethome3d.io;
 
 import java.io.File;
 import java.io.FileFilter;
+import java.io.IOException;
 import java.math.BigDecimal;
 import java.net.MalformedURLException;
 import java.net.URL;
@@ -47,6 +48,7 @@ import com.eteks.sweethome3d.model.LightSource;
 import com.eteks.sweethome3d.model.Sash;
 import com.eteks.sweethome3d.model.UserPreferences;
 import com.eteks.sweethome3d.tools.ResourceURLContent;
+import com.eteks.sweethome3d.tools.TemporaryURLContent;
 import com.eteks.sweethome3d.tools.URLContent;
 
 /**
@@ -81,19 +83,22 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
     /**
      * The key for the icon file of a piece of furniture (mandatory). 
      * This icon file can be either the path to an image relative to classpath
-     * or an absolute URL. 
+     * or an absolute URL. It should be encoded in application/x-www-form-urlencoded  
+     * format if needed. 
      */
     ICON("icon"),
     /**
      * The key for the plan icon file of a piece of furniture (optional).
      * This icon file can be either the path to an image relative to classpath
-     * or an absolute URL. 
+     * or an absolute URL. It should be encoded in application/x-www-form-urlencoded  
+     * format if needed.
      */
     PLAN_ICON("planIcon"),
     /**
      * The key for the 3D model file of a piece of furniture (mandatory).
      * The 3D model file can be either a path relative to classpath
-     * or an absolute URL. 
+     * or an absolute URL.  It should be encoded in application/x-www-form-urlencoded  
+     * format if needed.
      */
     MODEL("model"),
     /**
@@ -255,7 +260,7 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
    * Creates a default furniture catalog read from resources and   
    * furniture plugin folder if <code>furniturePluginFolder</code> isn't <code>null</code>.
    */
-  public DefaultFurnitureCatalog(final UserPreferences preferences, 
+;  public DefaultFurnitureCatalog(final UserPreferences preferences, 
                                  File furniturePluginFolder) {
     Map<FurnitureCategory, Map<CatalogPieceOfFurniture, Integer>> furnitureHomonymsCounter = 
         new HashMap<FurnitureCategory, Map<CatalogPieceOfFurniture,Integer>>();
@@ -288,19 +293,8 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
         // from most recent to least recent
         Arrays.sort(pluginFurnitureCatalogFiles, Collections.reverseOrder());
         for (File pluginFurnitureCatalogFile : pluginFurnitureCatalogFiles) {
-          try {
-            // Try to load the properties file describing furniture catalog from current file  
-            URL pluginFurnitureCatalogUrl = pluginFurnitureCatalogFile.toURI().toURL();
-            readFurniture(ResourceBundle.getBundle(PLUGIN_FURNITURE_CATALOG_FAMILY, Locale.getDefault(), 
-                                                   new URLClassLoader(new URL [] {pluginFurnitureCatalogUrl})), 
-                pluginFurnitureCatalogUrl, null, furnitureHomonymsCounter, identifiedFurniture);
-          } catch (MalformedURLException ex) {
-            // Ignore file
-          } catch (MissingResourceException ex) {
-            // Ignore malformed furniture catalog
-          } catch (IllegalArgumentException ex) {
-            // Ignore malformed furniture catalog
-          }
+          // Try to load the properties file describing furniture catalog from current file  
+          readPluginFurnitureCatalog(pluginFurnitureCatalogFile, furnitureHomonymsCounter, identifiedFurniture);
         }
       }
     }
@@ -325,7 +319,6 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
 
     for (URL pluginFurnitureCatalogUrl : pluginFurnitureCatalogUrls) {
       try {
-        // Try do load the properties file describing furniture catalog from current file  
         readFurniture(ResourceBundle.getBundle(PLUGIN_FURNITURE_CATALOG_FAMILY, Locale.getDefault(), 
                 new URLClassLoader(new URL [] {pluginFurnitureCatalogUrl})), 
             pluginFurnitureCatalogUrl, furnitureResourcesUrlBase, furnitureHomonymsCounter, identifiedFurniture);
@@ -336,6 +329,44 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
       }
     }
   }
+
+  private static final Map<File,URL> pluginFurnitureCatalogUrlUpdates = new HashMap<File,URL>(); 
+  
+  /**
+   * Reads plug-in furniture catalog from the <code>pluginFurnitureCatalogFile</code> file. 
+   */
+  private void readPluginFurnitureCatalog(File pluginFurnitureCatalogFile,
+                                          Map<FurnitureCategory, Map<CatalogPieceOfFurniture, Integer>> furnitureHomonymsCounter, 
+                                          List<String> identifiedFurniture) {
+    try {
+      URL pluginFurnitureCatalogUrl = pluginFurnitureCatalogFile.toURI().toURL();
+      long urlModificationDate = pluginFurnitureCatalogFile.lastModified();
+      URL urlUpdate = pluginFurnitureCatalogUrlUpdates.get(pluginFurnitureCatalogUrl.toString());
+      if (pluginFurnitureCatalogFile.canWrite()
+          && (urlUpdate == null 
+              || urlUpdate.openConnection().getLastModified() < urlModificationDate)) {
+        // Copy updated resource URL content to a temporary file to ensure furniture added to home can safely 
+        // reference any file of the catalog file even if its content is changed afterwards
+        TemporaryURLContent contentCopy = TemporaryURLContent.copyToTemporaryURLContent(new URLContent(pluginFurnitureCatalogUrl));
+        URL temporaryFurnitureCatalogUrl = contentCopy.getURL();
+        pluginFurnitureCatalogUrlUpdates.put(pluginFurnitureCatalogFile, temporaryFurnitureCatalogUrl);
+        pluginFurnitureCatalogUrl = temporaryFurnitureCatalogUrl;
+      } else if (urlUpdate != null) {
+        pluginFurnitureCatalogUrl = urlUpdate;
+      }
+      
+      ResourceBundle resourceBundle = ResourceBundle.getBundle(PLUGIN_FURNITURE_CATALOG_FAMILY, Locale.getDefault(), 
+          new URLClassLoader(new URL [] {pluginFurnitureCatalogUrl}));      
+      readFurniture(resourceBundle, pluginFurnitureCatalogUrl, null, furnitureHomonymsCounter,
+          identifiedFurniture);
+    } catch (MissingResourceException ex) {
+      // Ignore malformed furniture catalog
+    } catch (IllegalArgumentException ex) {
+      // Ignore malformed furniture catalog
+    } catch (IOException ex) {
+      // Ignore unaccessible catalog
+    }
+  }
   
   /**
    * Reads furniture of a given catalog family from resources.
@@ -446,7 +477,7 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
     float elevation = getOptionalFloat(resource, PropertyKey.ELEVATION.getKey(index), 0);
     float [][] modelRotation = getModelRotation(resource, PropertyKey.MODEL_ROTATION.getKey(index));
     // By default creator is eTeks
-    String creator = getOptionalString(resource, PropertyKey.CREATOR.getKey(index), "eTeks");
+    String creator = getOptionalString(resource, PropertyKey.CREATOR.getKey(index), null);
     String id = getOptionalString(resource, PropertyKey.ID.getKey(index), null);
     boolean resizable = true;
     try {
@@ -581,7 +612,7 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
       // or an URL relative to resourceUrlBase if it's not null
       URL url;
       if (resourceUrlBase != null) {
-        if (contentFile.indexOf('!') < 0) {
+        if (contentFile.indexOf('!') < 0 || contentFile.startsWith("jar:")) {
           url = new URL(resourceUrlBase, contentFile);
         } else {
           url = new URL("jar:" + new URL(resourceUrlBase, contentFile));
