@@ -57,6 +57,7 @@ import javax.media.j3d.Texture;
 import javax.media.j3d.Transform3D;
 import javax.media.j3d.TransformGroup;
 import javax.media.j3d.TransparencyAttributes;
+import javax.vecmath.Color3f;
 import javax.vecmath.Point3d;
 import javax.vecmath.Vector3d;
 import javax.vecmath.Vector3f;
@@ -172,7 +173,7 @@ public class DAELoader extends LoaderBase implements Loader {
   private static class DAEHandler extends DefaultHandler {
     private final SceneBase scene;
     private final URL       baseUrl;
-    private final Stack<TransformGroup> parentTransformGroups = new Stack<TransformGroup>();
+    private final Stack<Group>   parentGroups = new Stack<Group>();
     private final Stack<String>  parentElements = new Stack<String>();
     private final StringBuilder  buffer = new StringBuilder();
     private final List<Runnable> postProcessingBinders = new ArrayList<Runnable>();
@@ -184,7 +185,9 @@ public class DAELoader extends LoaderBase implements Loader {
     private final Map<String, String> sampler2DIds = new HashMap<String, String>();
     private final Map<String, List<Geometry>> geometries = new HashMap<String, List<Geometry>>();
     private final Map<String, float []> sources = new HashMap<String, float []>();
-    private final Map<String, float []> vertices = new HashMap<String, float []>();
+    private final Map<String, float []> positions = new HashMap<String, float []>();
+    private final Map<String, float []> normals = new HashMap<String, float []>();
+    private final Map<String, float []> textureCoordinates = new HashMap<String, float []>();
     private final Map<Geometry, String> geometryAppearances = new HashMap<Geometry, String>();
     private final List<int []> facesAndLinesPrimitives = new ArrayList<int[]>();
     private final List<int []> polygonsPrimitives = new ArrayList<int[]>();
@@ -294,20 +297,36 @@ public class DAELoader extends LoaderBase implements Loader {
           this.meshSourceId = attributes.getValue("id");
         } else if ("mesh".equals(parent) && "vertices".equals(name)) {
           this.verticesId = attributes.getValue("id");
-        } else if (this.verticesId != null && "input".equals(name) 
-                   && "POSITION".equals(attributes.getValue("semantic"))) {
-          this.vertices.put(this.verticesId, this.sources.get(attributes.getValue("source").substring(1)));
+        } else if (this.verticesId != null && "input".equals(name)) {
+          String sourceAnchor = attributes.getValue("source").substring(1);
+          if ("POSITION".equals(attributes.getValue("semantic"))) {
+            this.positions.put(this.verticesId, this.sources.get(sourceAnchor));
+          } else if ("NORMAL".equals(attributes.getValue("semantic"))) {
+            this.normals.put(this.verticesId, this.sources.get(sourceAnchor));
+          } else if ("TEXCOORD".equals(attributes.getValue("semantic"))) {
+            this.textureCoordinates.put(this.verticesId, this.sources.get(sourceAnchor));
+          }
         } else if (this.verticesId == null && "input".equals(name)) {
           this.inputCount++;
+          String sourceAnchor = attributes.getValue("source").substring(1);
+          int offset = Integer.parseInt(attributes.getValue("offset"));
           if ("VERTEX".equals(attributes.getValue("semantic"))) {
-            this.geometryVertices = this.vertices.get(attributes.getValue("source").substring(1));
-            this.geometryVertexOffset = Integer.parseInt(attributes.getValue("offset"));
+            this.geometryVertices = this.positions.get(sourceAnchor);
+            this.geometryVertexOffset = offset;
+            if (this.geometryNormals == null) {
+              this.geometryNormals = this.normals.get(sourceAnchor);
+              this.geometryNormalOffset = offset;
+            }
+            if (this.geometryTextureCoordinates == null) {
+              this.geometryTextureCoordinates = this.textureCoordinates.get(sourceAnchor);
+              this.geometryTextureCoordinatesOffset = offset;
+            }
           } else if ("NORMAL".equals(attributes.getValue("semantic"))) {
-            this.geometryNormals = this.sources.get(attributes.getValue("source").substring(1));
-            this.geometryNormalOffset = Integer.parseInt(attributes.getValue("offset"));
-          } else if ("input".equals(name) && "TEXCOORD".equals(attributes.getValue("semantic"))) {
-            this.geometryTextureCoordinates = this.sources.get(attributes.getValue("source").substring(1));
-            this.geometryTextureCoordinatesOffset = Integer.parseInt(attributes.getValue("offset"));
+            this.geometryNormals = this.sources.get(sourceAnchor);
+            this.geometryNormalOffset = offset;
+          } else if ("TEXCOORD".equals(attributes.getValue("semantic"))) {
+            this.geometryTextureCoordinates = this.sources.get(sourceAnchor);
+            this.geometryTextureCoordinatesOffset = offset;
           }
         } else if ("triangles".equals(name)
                    || "trifans".equals(name)
@@ -325,15 +344,15 @@ public class DAELoader extends LoaderBase implements Loader {
         } 
       } else if ("visual_scene".equals(name)) {
         TransformGroup visualSceneGroup = new TransformGroup();
-        this.parentTransformGroups.push(visualSceneGroup);
+        this.parentGroups.push(visualSceneGroup);
         this.visualScenes.put(attributes.getValue("id"), visualSceneGroup);
       } else if ("node".equals(name)) {
         TransformGroup nodeGroup = new TransformGroup();
-        if (this.parentTransformGroups.size() > 0) {
+        if (this.parentGroups.size() > 0) {
           // Add node to parent node only for children nodes
-          this.parentTransformGroups.peek().addChild(nodeGroup);
+          this.parentGroups.peek().addChild(nodeGroup);
         }
-        this.parentTransformGroups.push(nodeGroup);
+        this.parentGroups.push(nodeGroup);
         this.nodes.put(attributes.getValue("id"), nodeGroup);
         String nodeName = attributes.getValue("name");
         if (nodeName != null) {
@@ -343,13 +362,15 @@ public class DAELoader extends LoaderBase implements Loader {
         String geometryInstanceUrl = attributes.getValue("url");
         if (geometryInstanceUrl.startsWith("#")) {
           final String geometryInstanceAnchor = geometryInstanceUrl.substring(1);
-          final TransformGroup parentTransformGroup = (TransformGroup)this.parentTransformGroups.peek();
+          final Group parentGroup = new Group();
+          this.parentGroups.peek().addChild(parentGroup);
+          this.parentGroups.push(parentGroup);
           this.postProcessingBinders.add(new Runnable() {
               public void run() {
                 // Resolve URL at the end of the document
                 for (Geometry geometry : geometries.get(geometryInstanceAnchor)) {
                   Shape3D shape = new Shape3D(geometry);
-                  parentTransformGroup.addChild(shape);
+                  parentGroup.addChild(shape);
                 }
               }
             });
@@ -358,7 +379,7 @@ public class DAELoader extends LoaderBase implements Loader {
         String nodeInstanceUrl = attributes.getValue("url");
         if (nodeInstanceUrl.startsWith("#")) {
           final String nodeInstanceAnchor = nodeInstanceUrl.substring(1);
-          final TransformGroup parentTransformGroup = (TransformGroup)this.parentTransformGroups.peek();
+          final Group parentTransformGroup = this.parentGroups.peek();
           this.postProcessingBinders.add(new Runnable() {
               public void run() {
                 // Resolve URL at the end of the document
@@ -377,7 +398,7 @@ public class DAELoader extends LoaderBase implements Loader {
         if (materialInstanceTarget.startsWith("#")) {
           final String materialInstanceAnchor = materialInstanceTarget.substring(1);
           final String materialInstanceSymbol = attributes.getValue("symbol");
-          final Group group = this.parentTransformGroups.peek();
+          final Group group = this.parentGroups.peek();
           this.postProcessingBinders.add(new Runnable() {
               public void run() {
                 updateShapeAppearance(group, 
@@ -466,10 +487,10 @@ public class DAELoader extends LoaderBase implements Loader {
         this.geometryId = null;
       } if (this.geometryId != null) {
         handleGeometryElementsEnd(name, parent);
-      } else if ("visual_scene".equals(name)) {
-        this.parentTransformGroups.pop();
-      } else if ("node".equals(name)) {
-        this.parentTransformGroups.pop();
+      } else if ("visual_scene".equals(name)
+              || "node".equals(name)
+              || "node".equals(parent) && "instance_geometry".equals(name)) {
+        this.parentGroups.pop();
       } else if ("matrix".equals(name)) {
         mulTransformGroup(new Transform3D(this.floats));
       } else if ("node".equals(parent) && "rotate".equals(name)) {
@@ -543,10 +564,14 @@ public class DAELoader extends LoaderBase implements Loader {
     private void handleImageElementsEnd(String name) throws SAXException {
       if ("init_from".equals(name)) {
         try {
-          BufferedImage textureImage = ImageIO.read(new URL(baseUrl, getCharacters()));
+          URL textureImageUrl = new URL(baseUrl, getCharacters());
+          BufferedImage textureImage = ImageIO.read(textureImageUrl);
           if (textureImage != null) {
             TextureLoader textureLoader = new TextureLoader(textureImage);
-            this.textures.put(this.imageId, textureLoader.getTexture());
+            Texture texture = textureLoader.getTexture();
+            // Keep in user data the URL of the texture image
+            texture.setUserData(textureImageUrl);
+            this.textures.put(this.imageId, texture);
           }
         } catch (IOException ex) {
           // Ignore images at other format or not found
@@ -596,9 +621,17 @@ public class DAELoader extends LoaderBase implements Loader {
         } else {
           transparencyValue = 0;
         }
+        Appearance appearance = this.effectAppearances.get(this.effectId);
         if (transparencyValue > 0) {
-          this.effectAppearances.get(this.effectId).setTransparencyAttributes(new TransparencyAttributes(
+          appearance.setTransparencyAttributes(new TransparencyAttributes(
               TransparencyAttributes.NICEST, transparencyValue)); // 0 means opaque in Java 3D
+        } else {
+          // Set default color if it doesn't exist yet
+          Color3f black = new Color3f();
+          if (appearance.getMaterial() == null) {
+            appearance.setMaterial(new Material(black, black, black, black, 1));
+            appearance.setColoringAttributes(new ColoringAttributes(black, ColoringAttributes.SHADE_GOURAUD));
+          }
         }
         this.transparentColor = null;
         this.transparency = null;
@@ -883,7 +916,7 @@ public class DAELoader extends LoaderBase implements Loader {
      * given <code>transformMultiplier</code>.
      */
     private void mulTransformGroup(Transform3D transformMultiplier) {
-      TransformGroup transformGroup = this.parentTransformGroups.peek();
+      TransformGroup transformGroup = (TransformGroup)this.parentGroups.peek();
       Transform3D transform = new Transform3D();
       transformGroup.getTransform(transform);
       transform.mul(transformMultiplier);
