@@ -27,7 +27,6 @@ import javax.media.j3d.BranchGroup;
 import javax.media.j3d.Geometry;
 import javax.media.j3d.GeometryArray;
 import javax.media.j3d.Group;
-import javax.media.j3d.ImageComponent;
 import javax.media.j3d.Link;
 import javax.media.j3d.Material;
 import javax.media.j3d.Node;
@@ -136,13 +135,12 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
             // Add piece model scene to a normalized transform group
             TransformGroup modelTransformGroup = 
                 ModelManager.getInstance().getNormalizedTransformGroup(modelRoot, modelRotation, 1);
-            modelTransformGroup.addChild(modelRoot);
-            updatePieceOfFurnitureModelNode(modelTransformGroup, ignoreDrawingMode, waitModelAndTextureLoadingEnd);            
+            updatePieceOfFurnitureModelNode(modelRoot, modelTransformGroup, ignoreDrawingMode, waitModelAndTextureLoadingEnd);            
           }
           
           public void modelError(Exception ex) {
             // In case of problem use a default red box
-            updatePieceOfFurnitureModelNode(getModelBox(Color.RED), ignoreDrawingMode, waitModelAndTextureLoadingEnd);            
+            updatePieceOfFurnitureModelNode(getModelBox(Color.RED), new TransformGroup(), ignoreDrawingMode, waitModelAndTextureLoadingEnd);            
           }
         });
   }
@@ -203,16 +201,14 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
                                            ((color >>> 8) & 0xFF) / 256f,
                                                    (color & 0xFF) / 256f);
       Material material = new Material(materialColor, new Color3f(), materialColor, materialColor, 64);
-      setMaterialAndTexture(filledModelNode, material, null, false, 
-          piece.getWidth(), piece.getDepth(), piece.getHeight(), null);
+      setMaterialAndTexture(filledModelNode, material, null, false, null, null);
     } else if (piece.getTexture() != null) {
-      Vector3f modelSize = ModelManager.getInstance().getSize(filledModelNode);
       setMaterialAndTexture(filledModelNode, null, piece.getTexture(), waitTextureLoadingEnd, 
-          piece.getWidth(), piece.getDepth(), piece.getHeight(), modelSize);
+          new Vector3f(piece.getWidth(), piece.getHeight(), piece.getDepth()),
+          ModelManager.getInstance().getSize(((Group)filledModelNode).getChild(0)));
     } else {
       // Set default material and texture of model
-      setMaterialAndTexture(filledModelNode, null, null, false, 
-          piece.getWidth(), piece.getDepth(), piece.getHeight(), null);
+      setMaterialAndTexture(filledModelNode, null, null, false, null, null);
     }
   }
 
@@ -243,13 +239,18 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
    */
   private void updatePieceOfFurnitureVisibility() {
     HomePieceOfFurniture piece = (HomePieceOfFurniture)getUserData();
-    HomeEnvironment.DrawingMode drawingMode = this.home.getEnvironment().getDrawingMode();
+    Node outlineModelNode = getOutlineModelNode();
+    HomeEnvironment.DrawingMode drawingMode;
+    if (outlineModelNode != null) {
+      drawingMode = this.home.getEnvironment().getDrawingMode(); 
+    } else {
+      drawingMode = null; 
+    }
     // Update visibility of filled model shapes
     setVisible(getFilledModelNode(), piece.isVisible()
         && (drawingMode == null
             || drawingMode == HomeEnvironment.DrawingMode.FILL 
             || drawingMode == HomeEnvironment.DrawingMode.FILL_AND_OUTLINE));
-    Node outlineModelNode = getOutlineModelNode();
     if (outlineModelNode != null) {
       // Update visibility of outline model shapes
       setVisible(outlineModelNode, piece.isVisible()
@@ -277,14 +278,18 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
   /**
    * Updates transform group children with <code>modelMode</code>.
    */
-  private void updatePieceOfFurnitureModelNode(Node modelNode, boolean ignoreDrawingMode,
+  private void updatePieceOfFurnitureModelNode(Node modelNode,
+                                               TransformGroup normalization,
+                                               boolean ignoreDrawingMode,
                                                boolean waitTextureLoadingEnd) {    
     BranchGroup modelBranch = new BranchGroup();
+    normalization.addChild(modelNode);
+    normalization.setCapability(ALLOW_CHILDREN_READ);
     // Add model node to branch group
-    modelBranch.addChild(modelNode);
+    modelBranch.addChild(normalization);
     if (!ignoreDrawingMode) {
       // Add outline model node 
-      modelBranch.addChild(createOutlineModelNode(modelNode));
+      modelBranch.addChild(createOutlineModelNode(normalization));
     }
 
     setModelCapabilities(modelBranch);
@@ -319,7 +324,7 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
    * Returns a clone of the given node with an outline appearance on its shapes.
    */
   private Node createOutlineModelNode(Node modelNode) {
-    Node node = modelNode.cloneTree();
+    Node node = ModelManager.getInstance().cloneNode(modelNode);
     setOutlineAppearance(node);
     return node;
   }
@@ -386,17 +391,17 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
    * with a given <code>material</code>. 
    */
   private void setMaterialAndTexture(Node node, Material material, HomeTexture texture, boolean waitTextureLoadingEnd,
-                                     float pieceWidth, float pieceDepth, float pieceHeight, Vector3f modelSize) {
+                                     Vector3f pieceSize, Vector3f modelSize) {
     if (node instanceof Group) {
       // Set material and texture of all children
       Enumeration<?> enumeration = ((Group)node).getAllChildren(); 
       while (enumeration.hasMoreElements()) {
         setMaterialAndTexture((Node)enumeration.nextElement(), material, texture, waitTextureLoadingEnd,
-            pieceWidth, pieceDepth, pieceHeight, modelSize);
+            pieceSize, modelSize);
       }
     } else if (node instanceof Link) {
       setMaterialAndTexture(((Link)node).getSharedGroup(), material, texture, waitTextureLoadingEnd,
-          pieceWidth, pieceDepth, pieceHeight, modelSize);
+          pieceSize, modelSize);
     } else if (node instanceof Shape3D) {
       final Shape3D shape = (Shape3D)node;
       String shapeName = (String)shape.getUserData();
@@ -411,9 +416,7 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
         // Use appearance user data to store shape default material
         DefaultMaterialAndTexture defaultMaterialAndTexture = (DefaultMaterialAndTexture)appearance.getUserData();
         if (defaultMaterialAndTexture == null) {
-          defaultMaterialAndTexture = new DefaultMaterialAndTexture(appearance.getMaterial(), 
-              appearance.getTransparencyAttributes(), appearance.getTexCoordGeneration(), 
-              appearance.getTexture(), appearance.getTextureAttributes());
+          defaultMaterialAndTexture = new DefaultMaterialAndTexture(appearance);
           appearance.setUserData(defaultMaterialAndTexture);
         }
         if (material != null && defaultMaterialAndTexture.getTexture() == null) {
@@ -428,8 +431,8 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
           appearance.setMaterial((Material)DEFAULT_TEXTURED_SHAPE_MATERIAL);
           TexCoordGeneration texCoordGeneration = new TexCoordGeneration(TexCoordGeneration.OBJECT_LINEAR,
               TexCoordGeneration.TEXTURE_COORDINATE_2,
-              new Vector4f(-pieceWidth / modelSize.x / texture.getWidth(), 0, 0, 0), 
-              new Vector4f(0, pieceHeight / modelSize.y / texture.getHeight(), pieceDepth / modelSize.z / texture.getHeight(), 0));
+              new Vector4f(-pieceSize.x / modelSize.x / texture.getWidth(), 0, 0, 0), 
+              new Vector4f(0, pieceSize.y / modelSize.y / texture.getHeight(), pieceSize.z / modelSize.z / texture.getHeight(), 0));
           appearance.setTexCoordGeneration(texCoordGeneration);
           appearance.setTextureAttributes(MODULATE_TEXTURE_ATTRIBUTES);
           TextureManager.getInstance().loadTexture(texture.getImage(), waitTextureLoadingEnd,
@@ -574,16 +577,6 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
     appearance.setCapability(Appearance.ALLOW_TEXTURE_WRITE);
     appearance.setCapability(Appearance.ALLOW_TEXTURE_ATTRIBUTES_READ);
     appearance.setCapability(Appearance.ALLOW_TEXTURE_ATTRIBUTES_WRITE);
-    Texture texture = appearance.getTexture();
-    if (texture != null && !texture.isLive()) {
-      texture.setCapability(Texture.ALLOW_FORMAT_READ);
-      texture.setCapability(Texture.ALLOW_FORMAT_READ);
-      for (ImageComponent image : texture.getImages()) {
-        if (!image.isLive()) {
-          image.setCapability(ImageComponent.ALLOW_FORMAT_READ);
-        }
-      }
-    }
     appearance.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_READ);
     appearance.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_WRITE);
     PolygonAttributes polygonAttributes = appearance.getPolygonAttributes();
@@ -617,16 +610,12 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
     private final Texture                texture;
     private final TextureAttributes      textureAttributes;
 
-    public DefaultMaterialAndTexture(Material material, 
-                                     TransparencyAttributes transparencyAttributes, 
-                                     TexCoordGeneration texCoordGeneration, 
-                                     Texture texture, 
-                                     TextureAttributes textureAttributes) {
-      this.material = material;
-      this.transparencyAttributes = transparencyAttributes;
-      this.texCoordGeneration = texCoordGeneration;
-      this.texture = texture;
-      this.textureAttributes = textureAttributes;      
+    public DefaultMaterialAndTexture(Appearance appearance) {
+      this.material = appearance.getMaterial();
+      this.transparencyAttributes = appearance.getTransparencyAttributes();
+      this.texCoordGeneration = appearance.getTexCoordGeneration();
+      this.texture = appearance.getTexture();
+      this.textureAttributes = appearance.getTextureAttributes();      
     }
     
     public Material getMaterial() {
