@@ -300,6 +300,7 @@ public class ModelManager {
    * @param content an object containing a model
    * @param modelObserver the observer that will be notified once the model is available
    *    or if an error happens
+   * @throws IllegalStateException if the current thread isn't the Event Dispatch Thread.  
    */
   public void loadModel(Content content,
                         ModelObserver modelObserver) {
@@ -312,73 +313,72 @@ public class ModelManager {
    * @param content an object containing a model
    * @param synchronous if <code>true</code>, this method will return only once model content is loaded
    * @param modelObserver the observer that will be notified once the model is available
-   *    or if an error happens. When the model is loaded asynchronously, this method should be called
-   *    in Event Dispatch Thread and observer will be notified in EDT, otherwise it will be notified 
-   *    in the same thread as the caller.
+   *    or if an error happens. When the model is loaded synchronously, the observer will be notified
+   *    in the same thread as the caller, otherwise the observer will be notified in the Event 
+   *    Dispatch Thread and this method must be called in Event Dispatch Thread too.
+   * @throws IllegalStateException if synchronous is <code>false</code> and the current thread isn't 
+   *    the Event Dispatch Thread.  
    */
   public void loadModel(final Content content,
                         boolean synchronous,
                         ModelObserver modelObserver) {
     BranchGroup modelRoot = this.loadedModelNodes.get(content);
-    if (modelRoot == null) {
-      if (synchronous) {
-        try {
-          modelRoot = loadModel(content);
-        } catch (IOException ex) {
-          modelObserver.modelError(ex);
-        }
-        // Store in cache a model node for future copies 
+    if (modelRoot != null) {
+      // Notify cached model to observer with a clone of the model
+      modelObserver.modelUpdated((BranchGroup)cloneNode(modelRoot));
+    } else if (synchronous) {
+      try {
+        modelRoot = loadModel(content);
+        // Store in cache model node for future copies 
         this.loadedModelNodes.put(content, (BranchGroup)modelRoot);
-      } else {
-        if (this.modelsLoader == null) {
-          this.modelsLoader = Executors.newFixedThreadPool(Runtime.getRuntime().availableProcessors());
-        }
-        List<ModelObserver> observers = this.loadingModelObservers.get(content);
-        if (observers != null) {
-          // If observers list exists, content model is already being loaded
-          // register observer for future notification
-          observers.add(modelObserver);
-        } else {
-          // Create a list of observers that will be notified once content model is loaded
-          observers = new ArrayList<ModelObserver>();
-          observers.add(modelObserver);
-          this.loadingModelObservers.put(content, observers);
-          
-          // Load the model in an other thread
-          this.modelsLoader.execute(new Runnable() {
-            public void run() {
-              try {
-                final BranchGroup loadedModel = loadModel(content);
-                // Update loaded models cache and notify registered observers
-                loadedModelNodes.put(content, loadedModel);
-                EventQueue.invokeLater(new Runnable() {
-                    public void run() {
-                      for (final ModelObserver observer : loadingModelObservers.remove(content)) {
-                        BranchGroup modelNode = (BranchGroup)cloneNode(loadedModel);
-                        observer.modelUpdated(modelNode);
-                      }
-                    }
-                  });
-              } catch (final IOException ex) {
-                synchronized (loadedModelNodes) {
-                  EventQueue.invokeLater(new Runnable() {
-                      public void run() {
-                        for (final ModelObserver observer : loadingModelObservers.remove(content)) {
-                          observer.modelError(ex);
-                        }
-                      }
-                    });
-                }                  
-              }
-            }
-          });
-        }
-        return;
+        modelObserver.modelUpdated((BranchGroup)cloneNode(modelRoot));
+      } catch (IOException ex) {
+        modelObserver.modelError(ex);
       }
-    } 
-    
-    // Notify cached model to observer with a clone of the model
-    modelObserver.modelUpdated((BranchGroup)cloneNode(modelRoot));
+    } else if (!EventQueue.isDispatchThread()) {
+      throw new IllegalStateException("Asynchronous call out of Event Dispatch Thread");
+    } else {  
+      if (this.modelsLoader == null) {
+        this.modelsLoader = Executors.newFixedThreadPool(Runtime.getRuntime().availableProcessors());
+      }
+      List<ModelObserver> observers = this.loadingModelObservers.get(content);
+      if (observers != null) {
+        // If observers list exists, content model is already being loaded
+        // register observer for future notification
+        observers.add(modelObserver);
+      } else {
+        // Create a list of observers that will be notified once content model is loaded
+        observers = new ArrayList<ModelObserver>();
+        observers.add(modelObserver);
+        this.loadingModelObservers.put(content, observers);
+        
+        // Load the model in an other thread
+        this.modelsLoader.execute(new Runnable() {
+          public void run() {
+            try {
+              final BranchGroup loadedModel = loadModel(content);
+              // Update loaded models cache and notify registered observers
+              loadedModelNodes.put(content, loadedModel);
+              EventQueue.invokeLater(new Runnable() {
+                  public void run() {
+                    for (final ModelObserver observer : loadingModelObservers.remove(content)) {
+                      observer.modelUpdated((BranchGroup)cloneNode(loadedModel));
+                    }
+                  }
+                });
+            } catch (final IOException ex) {
+              EventQueue.invokeLater(new Runnable() {
+                  public void run() {
+                    for (final ModelObserver observer : loadingModelObservers.remove(content)) {
+                      observer.modelError(ex);
+                    }
+                  }
+                });
+            }
+          }
+        });
+      }
+    }
   }
   
   /**
@@ -400,7 +400,7 @@ public class ModelManager {
         Appearance clonedAppearance = (Appearance)appearance.cloneNodeComponent(true);
         Texture texture = appearance.getTexture();
         if (texture != null) {
-          clonedAppearance.setTexture((Texture)texture.cloneNodeComponent(false));
+          clonedAppearance.setTexture(texture);
         }
         clonedShape.setAppearance(clonedAppearance);
       }
@@ -424,7 +424,7 @@ public class ModelManager {
         Group group = (Group)node;
         Group clonedGroup = (Group)clonedNode;
         for (int i = 0, n = group.numChildren(); i < n; i++) {
-          Node clonedChild = cloneNode(group.getChild(i));
+          Node clonedChild = cloneNode(group.getChild(i), clonedSharedGroups);
           clonedGroup.addChild(clonedChild);
         }
       }
@@ -519,7 +519,7 @@ public class ModelManager {
         updateShapeNamesAndWindowPanesTransparency(scene);
         
         // Turn off lights because some loaders don't take into account the ~LOAD_LIGHT_NODES flag
-        turnOffLightsAndModulateTextures(modelNode);
+        turnOffLightsShareAndModulateTextures(modelNode);
 
         return modelNode;
       } catch (IllegalArgumentException ex) {
@@ -541,7 +541,9 @@ public class ModelManager {
       }
     }
     
-    if (lastException instanceof IncorrectFormatException) {
+    if (lastException instanceof IOException) {
+      throw (IOException)lastException;
+    } else if (lastException instanceof IncorrectFormatException) {
       IOException incorrectFormatException = new IOException("Incorrect format");
       incorrectFormatException.initCause(lastException);
       throw incorrectFormatException;
@@ -550,7 +552,9 @@ public class ModelManager {
       incorrectFormatException.initCause(lastException);
       throw incorrectFormatException;
     } else {
-      throw (IOException)lastException;
+      IOException otherException = new IOException();
+      otherException.initCause(lastException);
+      throw otherException;
     } 
   }  
   
@@ -582,15 +586,15 @@ public class ModelManager {
    * Turns off light nodes of <code>node</code> children, 
    * and modulates textures if needed.
    */
-  private void turnOffLightsAndModulateTextures(Node node) {
+  private void turnOffLightsShareAndModulateTextures(Node node) {
     if (node instanceof Group) {
       // Enumerate children
       Enumeration<?> enumeration = ((Group)node).getAllChildren(); 
       while (enumeration.hasMoreElements()) {
-        turnOffLightsAndModulateTextures((Node)enumeration.nextElement());
+        turnOffLightsShareAndModulateTextures((Node)enumeration.nextElement());
       }
     } else if (node instanceof Link) {
-      turnOffLightsAndModulateTextures(((Link)node).getSharedGroup());
+      turnOffLightsShareAndModulateTextures(((Link)node).getSharedGroup());
     } else if (node instanceof Light) {
       ((Light)node).setEnable(false);
     } else if (node instanceof Shape3D) {
@@ -598,10 +602,11 @@ public class ModelManager {
       if (appearance != null) {
         Texture texture = appearance.getTexture();
         if (texture != null) {
-          // Use nicest rendering
-          texture.setMinFilter(Texture.NICEST);
-          texture.setMagFilter(Texture.NICEST);
-          
+          // Share textures data as much as possible
+          Texture sharedTexture = TextureManager.getInstance().shareTexture(texture);
+          if (sharedTexture != texture) {
+            appearance.setTexture(sharedTexture);
+          }
           TextureAttributes textureAttributes = appearance.getTextureAttributes();
           if (textureAttributes == null) {
             // Mix texture and shape color
