@@ -43,6 +43,7 @@ import java.io.File;
 import java.io.IOException;
 import java.net.MalformedURLException;
 import java.net.URL;
+import java.net.URLEncoder;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.List;
@@ -995,7 +996,8 @@ public class ImportedFurnitureWizardStepsPanel extends JPanel
                     int slashIndex = entryName.lastIndexOf('/');
                     String entryFileName = entryName.substring(++slashIndex);
                     if (!entryFileName.startsWith(".")) {
-                      URL entryUrl = new URL("jar:" + urlContent.getURL() + "!/" + entryName);
+                      URL entryUrl = new URL("jar:" + urlContent.getURL() + "!/" 
+                          + URLEncoder.encode(entryName, "UTF-8").replace("+", "%20").replace("%2F", "/"));
                       modelContent = new TemporaryURLContent(entryUrl);
                       model = readModel(modelContent);
                       modelSize = ModelManager.getInstance().getSize(model);
@@ -1084,9 +1086,10 @@ public class ImportedFurnitureWizardStepsPanel extends JPanel
       if (!objFile.toLowerCase().endsWith(".obj")) {
         objFile += ".obj";
       }
-      File tempZipFile = File.createTempFile("urlContent", "tmp");
+      File tempZipFile = OperatingSystem.createTemporaryFile("import", ".zip");
       OBJWriter.writeNodeInZIPFile(model, tempZipFile, 0, objFile, "3D model import " + modelName);
-      return new TemporaryURLContent(new URL("jar:" + tempZipFile.toURI().toURL() + "!/" + objFile));
+      return new TemporaryURLContent(new URL("jar:" + tempZipFile.toURI().toURL() + "!/" 
+          + URLEncoder.encode(objFile, "UTF-8").replace("+", "%20")));
     } finally {
       setDefaultState();
     }
@@ -1518,7 +1521,8 @@ public class ImportedFurnitureWizardStepsPanel extends JPanel
 
     @Override
     public Dimension getPreferredSize() {
-      return new Dimension(128, 128);
+      Insets insets = getInsets();
+      return new Dimension(128 + insets.left + insets.right, 128  + insets.top + insets.bottom);
     }
 
     /**
